@@ -1,5 +1,5 @@
 #!/bin/sh
-exec perl -w -x $0 ${1+"$@"} # -*- mode: perl; perl-indent-level: 2; -*-
+exec perl -w -x "$0" ${1+"$@"} # -*- mode: perl; perl-indent-level: 2; -*-
 #!perl -w
 
 
@@ -9,8 +9,8 @@ exec perl -w -x $0 ${1+"$@"} # -*- mode: perl; perl-indent-level: 2; -*-
 ###                                                        ###
 ##############################################################
 
-## $Revision: 2.59 $
-## $Date: 2005-05-18 05:34:34 $
+## $Revision: 2.73 $
+## $Date: 2011/11/12 01:27:48 $
 ## $Author: kfogel $
 ##
 
@@ -143,7 +143,7 @@ use User::pwent    qw( getpwnam );
 # Globals --------------------------------------------------------------------
 
 # In case we have to print it out:
-my $VERSION = '$Revision: 2.59 $';
+my $VERSION = '$Revision: 2.73 $';
 $VERSION =~ s/\S+\s+(\S+)\s+\S+/$1/;
 
 ## Vars set by options:
@@ -216,6 +216,9 @@ my $After_Header = " ";
 # XML Encoding
 my $XML_Encoding = '';
 
+# XML Stylesheet file
+my $XML_Stylesheet = '';
+
 # Format more for programs than for humans.
 my $XML_Output = 0;
 my $No_XML_Namespace = 0;
@@ -225,6 +228,9 @@ my $No_XML_ISO_Date = 0;
 # ChangeLog style.
 my $FSF_Style = 0;
 
+# Set iff output should be like an FSF-style ChangeLog.
+my $FSF_Output = 0;
+
 # Show times in UTC instead of local time
 my $UTC_Times = 0;
 
@@ -839,11 +845,15 @@ sub output_header {
   my $version     = 'version="1.0"';
   my $declaration =
     sprintf '<?xml %s?>', join ' ', grep length, $version, $encoding;
+  my $stylesheet  =
+    length $XML_Stylesheet ?
+       sprintf '<?xml-stylesheet type="text/xsl" href="%s"?>', $XML_Stylesheet :
+       '';
   my $root        =
     $No_XML_Namespace ?
-      '<changelog>'     :
-        '<changelog xmlns="http://www.red-bean.com/xmlns/cvs2cl/">';
-  print $fh "$declaration\n\n$root\n\n";
+      '<changelog>'   :
+      '<changelog xmlns="http://www.red-bean.com/xmlns/cvs2cl/">';
+  print $fh "$declaration\n$stylesheet\n\n$root\n\n";
 }
 
 # -------------------------------------
@@ -968,6 +978,7 @@ sub output_tagdate {
 
   print $fh "<tagdate>\n";
   print $fh "<tagisodate>$isoDate</tagisodate>\n";
+  $tag = $self->escape($tag);     # more paranoia
   print $fh "<tagdatetag>$tag</tagdatetag>\n";
   print $fh "</tagdate>\n\n";
   return;
@@ -1186,7 +1197,12 @@ my $self = shift; my $class = ref $self;
 
     if (! $Output_To_Stdout) {
       $logfile_here =  $dir . $Log_File_Name;
-      $logfile_here =~ s/^\.\/\//\//;   # fix any leading ".//" problem
+      if (!$^O =~ /Win32/i) {
+        $logfile_here =~ s/^\.\/\//\//;   # fix any leading ".//" problem
+      }
+      else {
+        $logfile_here =~ s/^\.\/+//;      # remove any leading "./"
+      }
       $tmpfile      = "${logfile_here}.cvs2cl$$.tmp";
       $logfile_bak  = "${logfile_here}.bak";
 
@@ -1935,27 +1951,35 @@ sub grand_poobah { $_[0]->{grand_poobah} }
 sub read_changelog {
   my ($self, $command) = @_;
 
-  local (*READER, *WRITER);
+  local (*READER);
   my $pid;
   if (! $Input_From_Stdin) {
-    pipe(READER, WRITER)
-      or die "Couldn't form pipe: $!\n";
-    $pid = fork;
-    die "Couldn't fork: $!\n"
-      if ! defined $pid;
-    if ( ! $pid ) { # child
-      open STDOUT, '>&=' . fileno WRITER
-        or die "Couldn't dup stderr to ", fileno WRITER, "\n";
-      # strangely, some perls give spurious warnings about STDIN being opened
-      # for output only these close calls precede the STDOUT reopen above.
-      # I think they must be reusing fd 1.
-      close READER;
-      close STDIN;
-
-      exec @$command;
+    if ($^O =~ /Win32/i) {
+      open (READER, "@$command |")
+        or die "unable to run \"@$command\"";
     }
+    else {
+      local (*WRITER);
+      pipe(READER, WRITER)
+        or die "Couldn't form pipe: $!\n";
+      $pid = fork;
+      if (! defined $pid) {
+        die "Couldn't fork: $!\n";
+      }
+      if ( ! $pid ) { # child
+        open STDOUT, '>&=' . fileno WRITER
+          or die "Couldn't dup stderr to ", fileno WRITER, "\n";
+        # strangely, some perls give spurious warnings about STDIN being opened
+        # for output only these close calls precede the STDOUT reopen above.
+        # I think they must be reusing fd 1.
+        close READER;
+        close STDIN;
 
-    close WRITER;
+        exec @$command;
+      }
+
+      close WRITER;
+    }
 
     &main::debug ("(run \"@$command\")\n");
   }
@@ -2256,10 +2280,10 @@ sub read_date_author_and_state {
       $fullname = '';
     }
     for (grep defined, $fullname, $office, $workphone, $homephone) {
-      s/&/ucfirst(lc($pw->name))/ge;
+      s/&/ucfirst(lc($pw))/ge;
     }
     $author = $fullname . "  <" . $email . ">"
-      if $fullname ne '';
+      if defined $fullname && $fullname ne '';
   }
 
   $self->{rev_state}  = $state;
@@ -2360,10 +2384,10 @@ sub delta_check {
     unless $Delta_Mode;
 
   $Delta_StartTime = $time
-    if $time > $Delta_StartTime and grep { $_ eq $Delta_From } @$tags;
+    if $time > $Delta_StartTime and $Delta_From and grep { $_ eq $Delta_From } @$tags;
 
   $Delta_EndTime = $time
-    if $time > $Delta_EndTime and grep { $_ eq $Delta_To } @$tags;
+    if $time > $Delta_EndTime and $Delta_To and grep { $_ eq $Delta_To } @$tags;
 }
 
 sub run_ext {
@@ -2489,7 +2513,6 @@ sub common_path_prefix {
 }
 
 # -------------------------------------
-
 sub parse_options {
   # Check this internally before setting the global variable.
   my $output_file;
@@ -2538,6 +2561,7 @@ sub parse_options {
              'follow|F=s'     => \@Follow_Branches,
              'follow-only=s'  => \@Follow_Only,
              'xml-encoding=s' => \$XML_Encoding,
+             'xml-stylesheet=s' => \$XML_Stylesheet,
              'xml'            => \$XML_Output,
              'noxmlns'        => \$No_XML_Namespace,
              'no-xml-iso-date' => \$No_XML_ISO_Date,
@@ -2580,7 +2604,8 @@ sub parse_options {
              'delta=s'        => sub {
                my $arg = $_[1];
                if ( $arg =~
-                    /^([A-Za-z][A-Za-z0-9_\-\]\[]*):([A-Za-z][A-Za-z0-9_\-\]\[]*)$/ ) {
+                    /^([A-Za-z][A-Za-z0-9_\-\]\[\.]*)?:([A-Za-z][A-Za-z0-9_\-\]\[\.]*)?$/ )
+               {
                  $Delta_From = $1;
                  $Delta_To = $2;
                  $Delta_Mode = 1;
@@ -2590,6 +2615,7 @@ sub parse_options {
              },
 
              'FSF'             => sub {
+               $FSF_Output = 1;
                $Show_Times = 0;
                $Common_Dir = 0;
                $No_Extra_Indent = 1;
@@ -2668,6 +2694,11 @@ sub parse_options {
     $exit_with_admonishment = 1;
   }
 
+  if ($FSF_Output && $Cumulative) {
+    print STDERR "cannot pass both --FSF and --accum\n";
+    $exit_with_admonishment = 1;
+  }
+
   # Other consistency checks and option-driven logic
 
   # Bleargh.  Compensate for a deficiency of custom wrapping.
@@ -2858,7 +2889,7 @@ Use this if log data is in FSF ChangeLog style.
 
 =item B<--FSF>
 
-Attempt strict FSF-standard compatible output.
+Attempt strict FSF-standard compatible output (incompatible with B<--accum>).
 
 =item B<-W> I<SECS>, B<--window> I<SECS>
 
@@ -2920,6 +2951,11 @@ entry) are not shown.  This flag reverses that action.
 
 Blank line between each header and log message.
 
+=item B<--group-within-date>
+
+Group ChangeLog entries on the same date together, instead of having a
+separate entry for each commit on that date.
+
 =item B<--summary>
 
 Add CVS change summary information.
@@ -2938,7 +2974,7 @@ Show times in GMT/UTC instead of local time.
 
 =item B<--accum>
 
-Add to an existing ChangeLog (incompatible with B<--xml>).
+Add to an existing ChangeLog (incompatible with B<--xml> and B<--FSF>).
 
 =item B<-w>, B<--day-of-week>
 
@@ -2958,12 +2994,16 @@ Get ChangeLog header from I<FILE> ("B<->" means stdin).
 
 =item B<--xml>
 
-Output XML instead of ChangeLog format.
+Output XML instead of ChangeLog format (incompatible with B<--accum>).
 
-=item B<--xml-encoding> I<ENCODING.>
+=item B<--xml-encoding> I<ENCODING>
 
 Insert encoding clause in XML header.
 
+=item B<--xml-stylesheet> I<FILE>
+
+Insert xml-stylesheet processing instruction with I<FILE> formatting stylesheet file path in XML header.
+
 =item B<--noxmlns>
 
 Don't include xmlns= attribute in root element.
@@ -3180,7 +3220,7 @@ Contributions from
 
 =head1 BUGS
 
-Please report bugs to C<bug-cvs2cl@red-bean.com>.
+Please report bugs to C<cvs2cl-reports {_AT_} red-bean.com>.
 
 =head1 PREREQUISITES
 
@@ -3197,9 +3237,9 @@ Version_Control/CVS
 
 =head1 COPYRIGHT
 
-(C) 2001,2002,2003,2004 Martyn J. Pearce <fluffy@cpan.org>, under the GNU GPL.
+(C) 2001,2002,2003,2004 Martyn J. Pearce, under the GNU GPL.
 
-(C) 1999 Karl Fogel <kfogel@red-bean.com>, under the GNU GPL.
+(C) 1999 Karl Fogel, under the GNU GPL.
 
 cvs2cl.pl is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
